-- Testbench circuit for VOTE3SPA
--    (c) J. Dean Brock, 2004
--    EXCEPT for CONV_STD_LOGIC function taken from
--      John Wakerly, Digital Design: Principles and Practices 
library ieee ;
use ieee.std_logic_1164.all ;

entity VOTE3SPA_tb is
    generic (whocares : boolean := true) ;
end VOTE3SPA_tb ;
  
architecture TESTBENCH of VOTE3SPA_tb is
  component VOTE3SPA is
    port (
        Reset:  in  STD_LOGIC;
        Clock:  in  STD_LOGIC;
        SerIn:  in  STD_LOGIC;
        SerOut: out STD_LOGIC
    );
  end component ; 

  constant period : time := 200 ns ;
  
  signal TestReset  : STD_LOGIC ;
  signal TestClock  : STD_LOGIC ;
  signal TestSerIn  : STD_LOGIC ;
  signal TestSerOut : STD_LOGIC ;

  -- The CONV_STD_LOGIC function is taken from
  --      John Wakerly, Digital Design: Principles and Practices 
  function CONV_STD_LOGIC_VECTOR(ARG: integer; SIZE: integer)
                     return STD_LOGIC_VECTOR is
    variable result: STD_LOGIC_VECTOR (SIZE-1 downto 0) ;
    variable temp: integer ;
  begin
    temp := ARG ;
    for i in 0 to SIZE-1 loop
      if (temp mod 2) = 1 then
        result(i) := '1' ;
      else
        result(i) := '0' ;
      end if ;
      temp := temp / 2 ;
    end loop ;
    return result ;
  end ; 

  
begin -- TESTBENCH
  TestMod: VOTE3SPA
    port map (
      Reset  => TestReset ,
      Clock  => TestClock ,
      SerIn  => TestSerIn ,
      SerOut => TestSerOut
    ) ;
    
  TestDrive: process
    VARIABLE TestVect   : STD_LOGIC_VECTOR (0 to 2) ;
  begin
    TestReset <= '0' ;
    TestSerIn <= '0' ;
    TestClock <= '0' ;
    wait for period ;
    TestReset <= '1' ;
    wait for period ;
    TestReset <= '0' ;
    wait for period ;
    while (true) loop
       for i in 0 to 7 loop
       	 TestVect := CONV_STD_LOGIC_VECTOR(i, 3) ;
       	 for j in 0 to 2 loop
       	   TestSerIn <= TestVect(j) ;
       	   wait for period/4 ;
       	   TestClock <= '1' ;
       	   wait for period/2 ;
           TestClock <= '0' ;
       	   wait for period/4 ;
       	 end loop ;
       end loop ;
    end loop ;

  end process ;
  
end TESTBENCH ;





